// 006 u_c_help_handler_13
// 17 oct 2004

// -- Delphi implementation from the book:
// --   Design Patterns Elements of Reusable Object-Oriented Software
// --   Addison Wesley - 1994 - ISBN 0-201-63361-2
// -- (C) Erich GAMMA, Richard HELM, Ralph JOHNSON, John VLISSIDES 1994

// -- translation from C to Delphi performed by
// --   John COLIBRI  http://www.jcolibri.com

(*$r+*)

unit u_c_help_handler_13;
  interface

    type t_help_topic= Integer;
    const k_no_help_topic: t_help_topic= -1;

    type // -- abstract
         c_help_handler= class
                           _m_c_next_help_handler: c_help_handler;
                           _m_help_topic: t_help_topic;

                           Constructor create_help_handler(p_c_help_handler: c_help_handler;
                               p_help_topic: t_help_topic);
                           function f_has_help: Boolean; Virtual;
                           procedure set_handler(p_c_help_handler: c_help_handler;
                               p_help_topic: t_help_topic); Virtual;
                           procedure handle_help; Virtual;
                           Destructor Destroy; Override;
                         end; // c_help_handler

         // -- abstract
         c_widget= Class(c_help_handler)
                     _m_c_parent_widget: c_widget;

                     constructor create_widget(p_c_parent_widget: c_widget; p_help_topic: t_help_topic);
                     Destructor Destroy; Override;
                   end; // c_widget

         c_button= Class(c_widget)
                     constructor create_button(p_c_parent_widget: c_widget; p_help_topic: t_help_topic);
                     procedure handle_help; Override;
                     // -- all other operations that c_button overrides ...

                     Destructor Destroy; Override;
                   end; // c_button

         c_dialog= Class(c_widget)
                     // -- CAUTION: successor is not parent_widget but c_help_handler
                     constructor create_dialog(p_c_help_handler: c_help_handler; p_help_topic: t_help_topic);
                     procedure handle_help; Override;
                     // -- all other operations that c_dialog overrides ...

                     Destructor Destroy; Override;
                   end; // c_dialog

         c_application= Class(c_help_handler)
                          constructor create_application(p_help_topic: t_help_topic);
                          procedure handle_help; Override;
                          Destructor Destroy; Override;
                        end; // c_application

  implementation

    // -- c_help_handler

    Constructor c_help_handler.create_help_handler(p_c_help_handler: c_help_handler;
        p_help_topic: t_help_topic);
      begin
        Inherited Create;

        _m_c_next_help_handler:= p_c_help_handler;
        _m_help_topic:= p_help_topic;
      end; // create_help_handler

    function c_help_handler.f_has_help: boolean;
      begin
        Result:= _m_help_topic<> k_no_help_topic;
      end; // f_has_help

    procedure c_help_handler.set_handler(p_c_help_handler: c_help_handler;
        p_help_topic: t_help_topic);
      begin
      end; // set_handlerp_c_help_handler

    procedure c_help_handler.handle_help;
      begin
        if _m_c_next_help_handler<> Nil
          then _m_c_next_help_handler.handle_help;
      end; // handle_help

    Destructor c_help_handler.Destroy;
      begin
        Inherited;
      end; // Destroy

    // -- c_widget

    constructor c_widget.create_widget(p_c_parent_widget: c_widget; p_help_topic: t_help_topic);
      begin
        Inherited create_help_handler(Nil, p_help_topic);

        _m_c_parent_widget:= p_c_parent_widget;
      end; // create_widget

    Destructor c_widget.Destroy;
      begin
        Inherited;
      end; // Destroy

    // -- c_button

    constructor c_button.create_button(p_c_parent_widget: c_widget; p_help_topic: t_help_topic);
      begin
        Inherited create_widget(p_c_parent_widget, p_help_topic);
      end; // create_button

    procedure c_button.handle_help;
      begin
        if f_has_help
          then // display the button help
          else inherited handle_help;
      end; // handle_help

    Destructor c_button.Destroy;
      begin
        Inherited;
      end; // Destroy

    // -- c_dialog

    constructor c_dialog.create_dialog(p_c_help_handler: c_help_handler; p_help_topic: t_help_topic);
      begin
        Inherited create_widget(Nil, p_help_topic);
        set_handler(p_c_help_handler, p_help_topic);
      end; // create_dialog

    procedure c_dialog.handle_help;
      begin
        if f_has_help
          then // display the dialog help
          else inherited handle_help;
      end; // handle_help

    Destructor c_dialog.Destroy;
      begin
        Inherited;
      end; // Destroy

    // -- c_application

    constructor c_application.create_application(p_help_topic: t_help_topic);
      begin
        Inherited create_help_handler(Nil, p_help_topic);
      end; // create_application

    procedure c_application.handle_help;
      begin
        // -- show the list of help topics
      end; // handle_help

    Destructor c_application.Destroy;
      begin
        Inherited;
      end; // Destroy

    begin // u_c_help_handler
    end. // u_c_help_handler
